/***************************************************************************/
/*                                                                         */
/*  File: DrawModel.h                                                      */
/*  Date: 24-10-07                                                         */
/*                                                                         */
/***************************************************************************/
/*
	Simple class for rendering raw model data which we export and include
	using header files...
*/
/***************************************************************************/
#ifndef DRAWMODEL_H
#define DRAWMODEL_H


#include <d3dx9.h>				// Direct X Header Files

								// Direct X Library Files
#pragma comment(lib, "d3d9.lib")	
#pragma comment(lib, "d3dx9.lib")

// DirectX Device/Render Handles
extern LPDIRECT3D9			g_pD3D;			// main.cpp
extern LPDIRECT3DDEVICE9	g_pD3DDevice;	// main.cpp


#include "Maths.h"



class DrawModel
{
public:
	stModelData * m_modelData;

	IDirect3DVertexBuffer* m_pVertexBuffer;
	IDirect3DIndexBuffer*  m_pIndexBuffer;

	DWORD m_dwNumOfVertices;
	DWORD m_dwNumOfIndices;

	UINT D3DFVF_CUSTOMVERTEX; // (D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE);

	struct CUSTOMVERTEX
	{
		FLOAT x, y, z;			//Vertex position 
		D3DXVECTOR3 n;			//Direction the vertex is facing(normal)
		DWORD colour;			//The vertex colour.
	};


	DrawModel()
	{
		m_modelData=NULL;
		m_pVertexBuffer=NULL;
		m_pIndexBuffer=NULL;
		m_dwNumOfVertices=0;
		m_dwNumOfIndices=0;
	}
	~DrawModel(){Release();}

	void Create(stModelData* md, IDirect3DDevice9* pDevice)
	{
		m_modelData = md;
		m_dwNumOfIndices  = md->numIndices*3;
		m_dwNumOfVertices = md->numVerts;

		D3DFVF_CUSTOMVERTEX =  D3DFVF_XYZ |D3DFVF_NORMAL|D3DFVF_DIFFUSE;


		pDevice->CreateVertexBuffer(m_dwNumOfVertices * sizeof(CUSTOMVERTEX),
									0, D3DFVF_CUSTOMVERTEX,
									D3DPOOL_DEFAULT, &m_pVertexBuffer,
									NULL);

		pDevice->CreateIndexBuffer( m_dwNumOfIndices * sizeof(WORD),
									0, D3DFMT_INDEX16, D3DPOOL_MANAGED,
									&m_pIndexBuffer,
									NULL);

		WORD *pIndices			= NULL;
		CUSTOMVERTEX *pVertex	= NULL;

		m_pVertexBuffer->Lock(0,0, (void**)&pVertex, 0);
		m_pIndexBuffer->Lock(0,m_dwNumOfIndices, (void**)&pIndices, 0);

		for (int indx=0; indx<(int)m_dwNumOfIndices; indx++)
		{
			pIndices[indx] = m_modelData->i[indx];
		}

		int i = 0;
		for (int indx=0; indx<(int)m_dwNumOfVertices; indx++)
		{
			pVertex[indx].x = 0.5f * m_modelData->v[i]; i++;
			pVertex[indx].z = 0.5f * m_modelData->v[i]; i++;
			pVertex[indx].y = 0.5f * m_modelData->v[i]; i++;
			pVertex[indx].colour = 0xff00ffff;
		}

		WORD* inds		  = (WORD*)m_modelData->i;
		D3DXVECTOR3* vert = (D3DXVECTOR3*)m_modelData->v;

		for (int i=0; i<(int)m_dwNumOfIndices; i+=3)
		{
			WORD iA = inds[i+0];
			WORD iB = inds[i+1];
			WORD iC = inds[i+2];

			D3DXVECTOR3 v0 = vert[iA];
			D3DXVECTOR3 v1 = vert[iB];
			D3DXVECTOR3 v2 = vert[iC];

			D3DXVECTOR3 vA = v1 - v0;
			D3DXVECTOR3 vB = v2 - v0;

			D3DXVECTOR3 vN;
			D3DXVec3Cross(&vN, &vA, &vB);
			D3DXVec3Normalize(&vN, &vN);

			pVertex[iA].n = vN;
			pVertex[iB].n = vN;
			pVertex[iC].n = vN;
		}
		
		m_pIndexBuffer->Unlock();
		m_pVertexBuffer->Unlock();
	}

	IDirect3DVertexBuffer* GetVertexBuffer()
	{
		return m_pVertexBuffer;
	}

	void Release()
	{
		if (m_pIndexBuffer)  m_pIndexBuffer->Release();
		if (m_pVertexBuffer) m_pVertexBuffer->Release();
	}

	void Render(IDirect3DDevice* pDevice)
	{
		if (!m_modelData) return;

		pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);

		D3DXMATRIX matWorld;
		D3DXMatrixIdentity(&matWorld);

		pDevice->SetTransform(D3DTS_WORLD, &matWorld);

		pDevice->SetFVF( D3DFVF_CUSTOMVERTEX );
		pDevice->SetVertexShader(NULL);
		pDevice->SetPixelShader(NULL);
		pDevice->SetStreamSource(0, m_pVertexBuffer, 0, sizeof(CUSTOMVERTEX));
		pDevice->SetIndices(m_pIndexBuffer);
		int numOfPolygons = m_dwNumOfIndices / 3;

		//Render the polygons from the index buffer
		pDevice->DrawIndexedPrimitive(	D3DPT_TRIANGLELIST, 
										0, 
										0,
										m_dwNumOfVertices, 
										0,
										numOfPolygons);
	}

};






#endif //DRAWMODEL_H


